#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
A CLI client for Komodo AI

.. currentmodule:: komodo_cli
.. moduleauthor:: kmushegi <mushegiani@gmail.com>
"""

from .version import __release__, __version__  # noqa
